unit ConfigUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, BPLBase, Menus, ImgList, StdCtrls, Buttons,
  ExtCtrls;

type
  TConfigForm = class(TForm)                   
    lv: TListView;
    ImageList: TImageList;
    Panel1: TPanel;
    btnCancel: TBitBtn;
    btnOK: TBitBtn;
    procedure lvDblClick(Sender: TObject);
    procedure lvChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormShow(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    { Private declarations }
    procedure ListObjects;
    function GetObjectItemIndex(obj: TAppObject): integer;
    procedure FillItem( item: TListItem; obj: TAppObject);
  public
    BPL: TDynBPLBase;
    ConfigObject: TAppObject;
    { Public declarations }
  end;

var
  ConfigForm: TConfigForm;

implementation

{$R *.dfm}

procedure TConfigForm.FillItem(item: TListItem; obj: TAppObject);
var
  i: integer;
begin
  // Add new Item
  if item = nil then
  begin
    item := lv.Items.Add;
    for i:=0 to lv.Columns.Count-2 do
      item.SubItems.Add('');
  end;

  item.Data := obj;
  item.Caption := obj.Caption;
  item.ImageIndex := -1;

  // ------------------------------------------------
  // Copy image from AppObject, if assigned
  // ------------------------------------------------

  if item.ImageIndex < 0 then
  begin
    if Assigned(obj.Glyph) and not obj.Glyph.Empty then
      item.ImageIndex := ImageList.AddMasked(obj.Glyph,obj.Glyph.TransparentColor)
    else if Assigned(obj.ImageList)
    and (obj.ImageIndex >= 0) then
      item.ImageIndex := ImageList.AddImage(obj.ImageList,obj.ImageIndex);
  end;

end;

function TConfigForm.GetObjectItemIndex(obj: TAppObject): integer;
var
  i: integer;
begin
  Result := -1;
  for i:=0 to lv.items.Count-1 do
  begin
    if lv.Items[i].Data = obj then
    begin
      Result := i;
      exit;
    end;
  end;
end;

procedure TConfigForm.ListObjects;
var
  obj: TAppObject;
  i: integer;
  li: TListItem;
begin
  obj := BPL.GetNextAppObject(nil,otConfig,'');
  while obj <> nil do
  begin
    i := GetObjectItemIndex(obj);
    if i < 0 then
    begin
      li := lv.Items.Add;
      for i:=0 to lv.Columns.Count-2 do
        li.SubItems.Add('');
    end else
      li := lv.Items[i];

    FillItem(li,obj);
    obj := BPL.GetNextAppObject(obj,otConfig,'');
  end;
end;

procedure TConfigForm.lvDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

procedure TConfigForm.lvChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  btnOK.Enabled := lv.SelCount > 0;
end;

procedure TConfigForm.FormShow(Sender: TObject);
begin
  ConfigObject := nil;
  lv.Clear;
  ImageList.Clear;

  if Assigned(BPL) then
    ListObjects;
end;

procedure TConfigForm.btnOKClick(Sender: TObject);
var
  item: TListItem;
begin
  item := lv.Selected;
  if item.Data <> nil then
    ConfigObject := TAppObject(item.Data);
end;

end.
