unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BPLBase, BPLServer, ExtCtrls, ComCtrls, Menus;

type
  TMainForm = class(TForm)
    BPLServer: TDynBPLServer;
    MainPanel: TPanel;
    procedure BPLServerAppObjects0Execute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BPLServerAppObjects2Execute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure WMSysCommand(var Msg: TMessage); message WM_SYSCOMMAND;
		procedure Start(var WinMsg: TMessage); message WM_USER;
		procedure OpenConfigPackages(var WinMsg: TMessage); message WM_USER+1;
    procedure OpenConfigDialog;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses ConfigUnit;

{$R *.dfm}
{$R MainApp_Icons.res}

const
  MID_Config  = WM_USER+10;
  MID_NEWLINE = WM_USER+11;


procedure TMainForm.BPLServerAppObjects0Execute(Sender: TObject);
begin
  // Prevent from being called directly from a viewer
  // or manager module, which can be unloaded in this
  // dialog (and so being destroyed...)
	PostMessage(Handle,WM_USER+1,0,0);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  SysMenu: THandle;
begin
  // Add item to system menu
  SysMenu := GetSystemMenu(Handle, False);
  InsertMenu(SysMenu, Word(-1), MF_SEPARATOR, MID_NEWLINE, '');
  InsertMenu(SysMenu, Word(-1), MF_BYPOSITION, MID_Config, 'Configuration');

  // determine the application config file
  if (ParamCount > 0)
  and FileExists(ParamStr(1)) then
  begin
    Caption := ChangeFileExt(ExtractFileName(ParamStr(1)),'');
    BPLServer.AppObjects.Items[3].Text := ParamStr(1);
  end else
  begin
    Caption := Application.Title;
    BPLServer.AppObjects.Items[3].Text :=
      ChangeFileExt(Application.ExeName,'.app');
  end;
  BPLServer.IniFileName := BPLServer.AppObjects.Items[3].Text;

  // register application objects
  BPLServer.AppObjects.RegisterObjects;

  // send start message
	PostMessage(Handle,WM_USER,0,0);
end;

procedure TMainForm.Start(var WinMsg: TMessage);
begin
  BPLServer.LoadFromIniFile;
  BPLServer.CallService('App.Started',nil,true)
end;

procedure TMainForm.WMSysCommand(var Msg: TMessage);
begin
  if Msg.wParam = MID_CONFIG then
    OpenConfigDialog;
  inherited;
end;

procedure TMainForm.OpenConfigDialog;
var
  obj: TAppObject;
begin
  obj := nil;
  with TConfigForm.Create(self) do
  begin
    try
      BPL := BPLServer;
      if ShowModal = mrOk then
        obj := ConfigObject;
    finally
      Free;
    end;
  end;
  if obj <> nil then
    obj.Execute;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  BPLServer.CallService('App.End',nil,true);
end;

procedure TMainForm.OpenConfigPackages(var WinMsg: TMessage);
begin
  BPLServer.ConfigDialog;
  BPLServer.SaveToIniFile;
end;

procedure TMainForm.BPLServerAppObjects2Execute(Sender: TObject);
begin
  if MessageDlg('Close Application ?', mtWarning, [mbYes,mbNo], 0) = mrYes then
    Close;
end;

end.
